#
# Copyright 2018 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""Gtk 3+ Inkscape 1.0 Extensions Manager GUI."""

import os
import sys
import webbrowser

from gtkme import Window, ChildWindow, TreeView, asyncme
from gtkme.pixmap import PixmapManager
from gtkme.main import Gtk

from ..utils import DATA_DIR, INKSCAPE_PROFILE
from ..remote import RemoteArchive

def tag(name, color='#EFEFEF', fgcolor='#333333'):
    return f'<span size=\'small\' font_family=\'monospace\' style=\'italic\' background=\'{color}\' foreground=\'{fgcolor}\'> {name} </span> '

class LocalTreeView(TreeView):
    """A List of locally installed packages"""
    def __init__(self, target, *args, **kwargs):
        self._target = target
        self._pixmaps = kwargs['pixmaps']
        super().__init__(*args, **kwargs)

    def get_name(self, item):
        return f"<big><b>{item.name}</b></big>\n<i>{item.summary}</i>"

    def setup(self, *args, **kwargs):
        """Setup the treeview with one or many columns manually"""
        def get_version(item):
            if item.version:
                return f"<i>{item.version}</i>"

        col = self.ViewColumn('Extensions Package', expand=True,\
            text=self.get_name, template=None,\
            icon=lambda item: self._pixmaps.get(item.get_icon(), ''),\
            pad=0, size=None)

        col._icon_renderer.set_property('ypad', 2) # pylint: disable=protected-access
        col._text_renderer.set_property('xpad', 8) # pylint: disable=protected-access

        self.ViewColumn('Version', expand=False, text=get_version, template=None, pad=6)
        self.ViewColumn('Author', expand=False, text='author', pad=6)

        if self._target.version_specific:
            self.ViewColumn('Inkscape', expand=False, text='target', pad=6)

        self.ViewSort(data=lambda item: item.name, ascending=True, contains='name')

class RemoteTreeView(LocalTreeView):
    """A List of remote packages for installation"""
    def get_installed(self, item):
        if item.installed is None:
            item.set_installed(False)
            for installed_item in self._target.list_installed():
                if item.ident == installed_item.ident:
                    item.set_installed(True)

        return self._pixmaps.get(['gtk-no', 'gtk-yes'][bool(item.installed)])

    def setup(self, *args, **kwargs):
        def get_star(item):
            star = 'star-none.svg'
            if item.stars > 10:
                star = 'star-lots.svg'
            elif item.stars > 1:
                star = 'star-some.svg'
            return self._pixmaps.get(star)

        super().setup(*args, **kwargs)
        self.ViewColumn('Stars', expand=True, text='stars', pad=0, size=16, icon=get_star)
        self.ViewColumn('Installed', expand=False, text=False, pad=4, size=16, icon=self.get_installed)
        self.ViewSort(data=lambda item: item.stars, ascending=True)


class ExtensionManagerWindow(Window):
    name = 'gui'

    def get_name(self, item):
        summary = item.summary
        if not item.verified:
            summary = tag("Unverified", "#AC3A3A", "white") + summary.strip()
        return f"<big><b>{item.name}</b></big>\n<i>{summary}</i>\n" + \
            ''.join(tag(t) for t in item.tags)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.target = self.gapp.opt.target

        self.pixmaps = PixmapManager('pixmaps', pixmap_dir=DATA_DIR, size=48, load_size=(96,96))
        self.searching = self.widget('dl-searching')
        self.searchbox = self.searching.get_parent()
        self.searchbox.remove(self.searching)

        self.local = LocalTreeView(
            self.target,
            self.widget('local_extensions'),
            pixmaps=self.pixmaps,
            selected=self.select_local)

        self.remote = RemoteTreeView(
            self.target,
            self.widget('remote_extensions'),
            pixmaps=self.pixmaps,
            selected=self.select_remote)

        self.widget('loading').start()
        self.window.show_all()

        self.refresh_local()
        self.widget('remote_install').set_sensitive(False)
        self.widget('remote_info').set_sensitive(False)
        self.widget('local_install').set_sensitive(True)
        self.window.set_title(self.target.label)

        if not self.target.is_search:
            self.widget('remote_getlist').show()
            self.widget('dl-search').hide()
        else:
            self.widget('dl-search').show()
            self.widget('remote_getlist').hide()

    def change_local_all(self, widget, unk):
        """When the source switch button is clicked"""
        self.refresh_local()

    def remote_getlist(self, widget):
        self._remote_search('')

    @asyncme.run_or_none
    def refresh_local(self):
        """Searches for locally installed extensions and adds them"""
        self.local.clear()
        filtered = not self.widget('local_all').get_active()
        self.widget('local_uninstall').set_sensitive(False)
        self.widget('local_information').set_sensitive(False)

        all_packages = []
        for item in self.target.list_installed(cached=False):
            self.local.add_item([item])
        self.widget('loading').stop()

    def select_local(self, item):
        """Select an installed extension"""
        self.widget('local_uninstall').set_sensitive(item.is_uninstallable())
        self.widget('local_information').set_sensitive(bool(item))

    def local_information(self, widget):
        """Show the more information window"""
        if self.local.selected:
            self.load_window('info', pixmaps=self.pixmaps, item=self.local.selected)

    def local_uninstall(self, widget):
        """Uninstall selected extection package"""
        item = self.local.selected
        if item.is_uninstallable():
            if item.uninstall():
                self.local.remove_item(item)
                self.remote.refresh()

    def change_remote_all(self, widget, unk):
        """When the source switch button is clicked"""
        self.remote_search(self.widget('dl-search'))

    def remote_search(self, widget):
        """Remote search activation"""
        query = widget.get_text()
        if len(query) > 2:
            self._remote_search(query)

    def _remote_search(self, query):
        filtered = self.widget('remote_target').get_active()
        self.remote.clear()
        self.widget('remote_install').set_sensitive(False)
        self.widget('remote_info').set_sensitive(False)
        self.widget('dl-search').set_sensitive(False)
        self.searchbox.add(self.searching)
        self.widget('dl-searching').start()
        self.async_search(query, filtered)

    def remote_info(self, widget):
        """Show the remote information"""
        if self.remote.selected and self.remote.selected.link:
            webbrowser.open(self.remote.selected.link)

    @asyncme.run_or_none
    def async_search(self, query, filtered):
        """Asyncronous searching in PyPI"""
        for package in self.target.search(query, filtered):
            self.add_search_result(package)
        self.search_finished()

    @asyncme.mainloop_only
    def add_search_result(self, package):
        """Adding things to Gtk must be done in mainloop"""
        self.remote.add_item([package])

    @asyncme.mainloop_only
    def search_finished(self):
        """After everything, finish the search"""
        self.searchbox.remove(self.searching)
        self.widget('dl-search').set_sensitive(True)
        self.replace(self.searching, self.remote)

    def select_remote(self, item):
        """Select an installed extension"""
        # Do we have a place to install packages to?
        active = self.remote.selected \
            and self.remote.selected.is_installable() \
            and not self.remote.selected.installed
        self.widget('remote_install').set_sensitive(active)
        if self.remote.selected.link:
            self.widget('remote_info').set_sensitive(True)

    def remote_install(self, widget):
        """Install a remote package"""
        self.widget('remote_install').set_sensitive(False)
        item = self.remote.selected
        try:
            item.install()
        except Exception as err:
            self.dialog(str(err))
            return
        self.remote.refresh()
        self.refresh_local()
        self.widget('main_notebook').set_current_page(0)

    def local_install(self, widget):
        """Install from a local filename"""
        dialog = Gtk.FileChooserDialog(
            title="Please choose a package file", transient_for=self.window,
            action=Gtk.FileChooserAction.OPEN)

        dialog.add_buttons(
            Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
            Gtk.STOCK_OPEN, Gtk.ResponseType.OK)

        filter_py = Gtk.FileFilter()
        filter_py.set_name("Packages")
        filter_py.add_mime_type("application/x-compressed-tar")
        filter_py.add_mime_type("application/zip")
        dialog.add_filter(filter_py)

        filter_text = Gtk.FileFilter()
        filter_text.set_name("Python Wheel")
        filter_text.add_mime_type("application/zip")
        filter_text.add_pattern("*.whl")
        dialog.add_filter(filter_text)

        filter_any = Gtk.FileFilter()
        filter_any.set_name("Any files")
        filter_any.add_pattern("*")
        dialog.add_filter(filter_any)

        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            self.target._install(dialog.get_filename(), {})
            self.refresh_local()

        dialog.destroy()

    def dialog(self, msg):
        self.widget('dialog_msg').set_label(msg)
        self.widget('dialog').set_transient_for(self.window)
        self.widget('dialog').show_all()

    def close_dialog(self, widget):
        self.widget('dialog_msg').set_label('')
        self.widget('dialog').hide()




#
# Copyright (C) 2019-2021 Martin Owens
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Searching for external packages and getting meta data about them.
"""

import re
import os
import json
import logging
import requests

from zipfile import ZipFile

from cachecontrol import CacheControl, CacheControlAdapter
from cachecontrol.caches.file_cache import FileCache
from cachecontrol.heuristics import ExpiresAfter

from inkex.command import CommandNotFound, ProgramRunError, call
from inkex.inkscape_env import get_bin
from collections import defaultdict

from .utils import INKSCAPE_VERSION, CACHE_DIR
from .package import DEFAULT_ICON, PackageItem

PYTHON_VERSION = 'py3'

class RemoteFile(object):
    """A remote file, icon, zip etc"""
    def __init__(self, session, url):
        self.session = session
        self.url = url

    def __str__(self):
        return self.url

    def get(self):
        return self.session.get(self.url)

    def filename(self):
        return self.url.split('/')[-1]

    def filepath(self):
        return os.path.join(CACHE_DIR, self.filename())

    def as_local_file(self):
        if self.url:
            remote = self.get()
            if remote and remote.status_code == 200:
                with open(self.filepath(), 'wb') as fhl:
                    fhl.write(remote.content)
                return self.filepath()
        return None

class RemoteArchive(object):
    """The remote archive used to be PyPI but is now inkscape's own website."""
    URL = 'https://inkscape.org/gallery/={category}/json/'

    def __init__(self, category, version=INKSCAPE_VERSION):
        self.version = version
        self.url = self.URL.format(category=category)
        self.session = requests.session()
        self.session.mount('https://', CacheControlAdapter(
            cache=FileCache(CACHE_DIR),
            heuristic=ExpiresAfter(days=5),
        ))

    def _remote_file(self, url):
        return RemoteFile(self.session, url)

    def search(self, query, filtered=True):
        """Search for extension packages"""
        for info in self._search(query):
            item = PackageItem(info, remote=self._remote_file)
            if not filtered or not self.version or self.version in item.targets:
                yield item

    def _search(self, query, tags=[]):
        """Search for the given query and yield each item, will raise if any other response"""
        response = self.session.get(self.url, params={'q': query, 'tags': tags})
        for item in response.json()['items']:
            yield item




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2018 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Inkscape Extensions Manager, Graphical User Interface (Gtk3)
"""

from inkex import inkscape_env

import os
import sys
import logging

from argparse import ArgumentParser

# The pinned version of pip has warnings for python 3.8
import warnings
with warnings.catch_warnings():
    warnings.filterwarnings("ignore", category=DeprecationWarning)
    import html5lib
    import imp

from inkman.target import BasicTarget, ExtensionsTarget
from inkman.gui import ManagerApp

TARGETS = [ 
    # Website slug, Visible name, local directory, search instead of list
    ExtensionsTarget('extension', 'Extensions', 'extensions', True, filters=("*.inx",)),
    BasicTarget('template', 'Templates', 'templates', True, filters=("*.svg",)),
    BasicTarget('palette', 'Shared Paletts', 'palettes', filters=("*.gpl",)),
    BasicTarget('symbol', 'Symbol Collections', 'symbols', filters=("*.svg",)),
    BasicTarget('keyboard', 'Keyboard Shortcuts', 'keys', filters=('*.xml',)),
    # ('marker', 'Marker Collections', '', False), # No marker config
    # ('pattern', 'Pattern Collections', '', False), # No pattern config
    # ('', 'User Interface Themes', 'themes', False), # No website category
    # ('', 'Paint Server', 'paint', False), # No website category
    # ('', 'User Interfaces', 'ui', False), # No website category
    # ('', 'Icon Sets', 'icons', False), # No website category
]

def run(args):
    arg_parser = ArgumentParser(description=__doc__)
    arg_parser.add_argument("input_file", nargs="?")
    arg_parser.add_argument('--target', default=TARGETS[0].category,
        choices=[t.category for t in TARGETS])
    options = arg_parser.parse_args(args)
    try:
        ManagerApp(start_loop=True,
            target=[t for t in TARGETS if t.category == options.target][0])
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")

if __name__ == '__main__':
    run(sys.argv[1:])



#
# Copyright 2018-2020 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""All graphical interfaces bound into a GtkApp"""

import os

from gtkme import GtkApp

from .main import ExtensionManagerWindow
from .info import MoreInformation

class ManagerApp(GtkApp):
    """Load the inkscape extensions glade file and attach to window"""
    glade_dir = os.path.join(os.path.dirname(__file__), '..', 'data')
    app_name = 'inkscape-extensions-manager'
    windows = [ExtensionManagerWindow, MoreInformation]



#!/usr/bin/env python3
# coding=utf-8
#
# Copyright (C) 2018-2019 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
"""
Setup the inkscape extensions manager (inkman)
"""

from setuptools import setup
from inkman import __version__, __pkgname__

setup(
    name=__pkgname__,
    version=__version__,
    description='Inkscape manager user interface (gtk)',
    long_description="See README for full details",
    author='Martin Owens',
    url='https://gitlab.com/inkscape/extension-manager',
    author_email='doctormo@gmail.com',
    platforms='linux',
    license='GPLv3',
    include_package_data=True,
    packages=['inkman', 'inkman.gui', 'inkman.data', 'inkman.data.pixmaps'],
    data_files=[
        ('', ['manage_extensions.inx']),
        ('', ['manage_extensions.py']),
    ],
    classifiers=[
        'Environment :: Plugins',
        'Topic :: Multimedia :: Graphics :: Editors :: Vector-Based',

        'Intended Audience :: Other Audience',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',

        'Development Status :: 5 - Production/Stable',
    ],
    install_requires=[
        'appdirs',
        'requests',
        'cachecontrol[filecache]',
        'gtkme>=1.5',
        'pip==20.1',
        'lxml',
    ],
)



#!/usr/bin/env python
#
# pylint: disable=invalid-name
"""
Call python extensions, but make sure all the right python paths are setup.

There are two ways of using this file, you can change the inx file so it
loads inkpath.py with a hidden param 'call' that specifies the Extension class
to load.

Alternatively you can import call before you import inkex or other modules
which your extension requires.
"""

import os
import sys
import site
import argparse
import importlib

def load_prefixes():
    """
    The goal is to treat any non-python path as if it's a prefix,
    this is because inkscape will pass us a pythonpath that won't include
    the site-packages that may be INSIDE the path, where the path was used
    as a PREFIX during installation by the inkman application.
    """
    sitepaths = []
    for syspath in sys.path:
        if 'python' not in syspath:
            if syspath.rstrip('/').endswith('/bin'):
                syspath = syspath.rstrip('/')[:-4]
            site.PREFIXES.append(syspath)
        if '-packages' in syspath:
            sitepaths.append(syspath)

    if '/bin/' in __file__:
        site.PREFIXES.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

    if hasattr(site, 'getsitepackages'):
        sitepaths = site.getsitepackages()
    for sitedir in sitepaths:
        sitedir = sitedir.replace('dist-packages', 'site-packages')
        if os.path.isdir(sitedir):
            site.addsitedir(sitedir, set())

if __name__ == '__main__':
    # Find every possible module location based on the sys bucket
    load_prefixes()

    # Steal the one argument we need and put the rest back.
    parser = argparse.ArgumentParser()
    parser.add_argument('--call')
    args, sys.argv = parser.parse_known_args()
    call = args.call

    if not call or '.' not in call:
        sys.stderr.write("Call must be specified and must contain a module.ClassName\n")
        sys.exit(5)

    modname, clsname = call.rsplit('.', 1)
    try:
        mod = importlib.import_module(modname)
        cls = getattr(mod, clsname)
    except ImportError as err:
        sys.stderr.write("Can't find extension file: {}\n".format(err))
        sys.exit(1)
    except AttributeError as err:
        sys.stderr.write("Can't find extension class: {}\n".format(err))
        sys.exit(2)

    cls().run()



#
# Copyright (C) 2019 Martin Owens
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Utilities functions for inkscape extension manager.
"""

import os
import sys

from collections import defaultdict
from email.parser import FeedParser
from appdirs import user_cache_dir

from inkex.inx import InxFile

CACHE_DIR = user_cache_dir('inkscape-extension-manager', 'Inkscape')
INKSCAPE_VERSION = os.environ.get('INKSCAPE_VERSION', None)
INKSCAPE_PROFILE = os.environ.get('INKSCAPE_PROFILE_DIR', None)

if not INKSCAPE_PROFILE and 'VIRTUAL_ENV' in os.environ:
    INKSCAPE_PROFILE = os.path.dirname(os.environ['VIRTUAL_ENV'])

if not INKSCAPE_PROFILE:
    sys.stderr.write("The Inkscape profile directory isn't set!\n")
    sys.exit(2)

DATA_DIR = os.path.join(os.path.dirname(__file__), 'data')
ICON_SEP = ('-' * 12) + 'SVGICON' + ('-' * 12)

def _pythonpath():
    for pth in os.environ.get('PYTHONPATH', '').split(':'):
        if os.path.isdir(pth):
            yield pth

def get_user_directory():
    """Return the user directory where extensions are stored."""
    if 'INKSCAPE_PROFILE_DIR' in os.environ:
        return os.path.abspath(
            os.path.expanduser(
                os.path.join(os.environ['INKSCAPE_PROFILE_DIR'], 'extensions')))

    home = os.path.expanduser("~")
    for pth in _pythonpath():
        if pth.startswith(home):
            return pth

def get_inkscape_directory():
    """Return the system directory where inkscape's core is."""
    for pth in _pythonpath():
        if os.path.isdir(os.path.join(pth, 'inkex')):
            return pth


def format_requires(string):
    """Get a version requires."""
    primary = string.split('; ', 1)[0]
    if '(' in primary:
        primary, version = primary.split('(', 1)
        return (primary.strip(), version.strip(') '))
    return (primary.strip(), None)

def parse_metadata(data):
    """
    Convert older email based meta data into a newer json format,
    See PEP 566 for details.
    """
    feed_parser = FeedParser()
    feed_parser.feed(data)
    metadata = feed_parser.close()

    def getdict():
        """Multi-dimentional dictionary"""
        return defaultdict(getdict)
    ret = defaultdict(getdict)

    ret['description'] = metadata.get_payload()
    ret['requires'] = [ format_requires(m)
        for m in metadata.get_all('Requires-Dist', [])
    ]

    for key, value in metadata.items():
        if key == 'Home-page':
            ret['extensions']['python.details']['project_urls']['Home'] = value
        elif key == 'Classifier':
            ret['classifiers'] = list(ret['classifiers'])
            ret['classifiers'].append(value)
        else:
            ret[key.lower().replace('-', '_')] = value

    return ret

def clean_author(data):
    """Clean the author so it has consistant keys"""
    for contact in data.get('extensions', {}).get('python.details', {}).get('contacts', []):
        if contact['role'] == 'author':
            data['author'] = contact['name']
            data['author_email'] = contact.get('email', '')
            if '<' in data['author']:
                data['author'], other = data['author'].split('<', 1)
                if '@' in other and not data['author_email']:
                    data['author_email'] = data['author'].split('>')[0]
    return data

class ExtensionInx(InxFile):
    """Information about an extension specifically"""
    ident = property(lambda self: super().ident or f"[no-id] {self.filename}")
    name = property(lambda self: super().name or f"[unnamed] {self.ident}")

    @property
    def menu(self):
        menu = self.xml.find_one('effect/effects-menu')
        if menu is not None and menu.get('hidden', "false") == "true":
            return ["_hidden", self.name]
        return super().menu
